/*
 *
 *      Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the pig4cloud.com developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: hnqz
 *
 */

package com.qunzhixinxi.hnqz.admin.api.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.qunzhixinxi.hnqz.admin.api.enums.SignatureStatusEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * <p>
 * 用户表
 * </p>
 *
 * @author hnqz
 * @since 2017-10-29
 */
@Data
@ApiModel(value = "用户")
public class SysUser implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * 主键ID
	 */
	@TableId(value = "user_id", type = IdType.AUTO)
	@ApiModelProperty(value = "主键id")
	private Integer userId;

	/**
	 * 用户名
	 */
	@ApiModelProperty(value = "用户名")
	private String username;

	/**
	 * 密码
	 */
	@ApiModelProperty(value = "密码")
	private String password;

	/**
	 * 身份证号
	 */
	@ApiModelProperty(value = "身份证号")
	private String idCardNumber;

	/**
	 * 签名
	 */
	@ApiModelProperty(value = "签名")
	private String userSign;

	/**
	 * 随机盐
	 */
	@JsonIgnore
	@ApiModelProperty(value = "随机盐")
	private String salt;

	/**
	 * 创建时间
	 */
	@ApiModelProperty(value = "创建时间")
	private LocalDateTime createTime;

	/**
	 * 修改时间
	 */
	@ApiModelProperty(value = "修改时间")
	private LocalDateTime updateTime;

	/**
	 * 0-正常，1-删除
	 */
	@TableLogic
	@ApiModelProperty(value = "删除标记,1:已删除,0:正常")
	private String delFlag;

	/**
	 * 锁定标记
	 */
	@ApiModelProperty(value = "锁定标记")
	private String lockFlag;

	/**
	 * 手机号
	 */
	@ApiModelProperty(value = "手机号")
	private String phone;

	/**
	 * 头像
	 */
	@ApiModelProperty(value = "头像地址")
	private String avatar;

	/**
	 * 部门ID
	 */
	@ApiModelProperty(value = "用户所属部门id")
	private Integer deptId;

	/**
	 * 租户ID
	 */
	@ApiModelProperty(value = "用户所属租户id")
	private Integer tenantId;

	/**
	 * 微信openid
	 */
	@ApiModelProperty(value = "微信openid")
	private String wxOpenid;

	/**
	 * 微信小程序openId
	 */
	@ApiModelProperty(value = "微信小程序openid")
	private String miniOpenid;

	/**
	 * QQ openid
	 */
	@ApiModelProperty(value = "QQ openid")
	private String qqOpenid;

	/**
	 * 码云唯一标识
	 */
	@ApiModelProperty(value = "码云唯一标识")
	private String giteeLogin;

	/**
	 * 开源中国唯一标识
	 */
	@ApiModelProperty(value = "开源中国唯一标识")
	private String oscId;

	/**
	 * 用户姓名
	 */
	@ApiModelProperty(value="用户姓名")
	private String realname;
	/**
	 * 省
	 */
	@ApiModelProperty(value="省")
	private String province;
	/**
	 * 省
	 */
	@ApiModelProperty(value="省")
	private String provinceCode;
	/**
	 * 市
	 */
	@ApiModelProperty(value="市")
	private String city;
	/**
	 * 市
	 */
	@ApiModelProperty(value="市")
	private String cityCode;
	/**
	 * 区
	 */
	@ApiModelProperty(value="区")
	private String area;
	/**
	 * 地址
	 */
	@ApiModelProperty(value="地址")
	private String address;
	/**
	 * 备用字段1
	 */
	@ApiModelProperty(value="备用字段1")
	private String temp1;
	/**
	 * 备用字段2
	 */
	@ApiModelProperty(value="备用字段2")
	private String temp2;
	/**
	 * 备用字段3
	 */
	@ApiModelProperty(value="备用字段3")
	private String temp3;
	/**
	 * 备用字段4
	 */
	@ApiModelProperty(value="备用字段4")
	private String temp4;
	/**
	 * 物明经销商管理平台ID
	 */
	@ApiModelProperty(value="备用字段5")
	private String temp5;

	/**
	 * 物明经销商管理平台ID
	 */
	@ApiModelProperty(value="备用字段5")
	private String platId;

	/**
	 * 物明经销商管理平台ID
	 */
	@ApiModelProperty(value="备用字段5")
	private String drugEntId;

	/**
	 * 物明经销商管理平台ID
	 */
	@ApiModelProperty(value="")
	private String w1;

	/**
	 * 税邦云认证状态
	 */
	@ApiModelProperty(value="税邦云认证状态")
	private Integer certStatus;

	/**
	 * 人力家认证结果描述
	 */
	@ApiModelProperty(value="人力家认证结果描述")
	private String certRemark;

	/**
	 * 用户认证类型
	 * 0-人力家
	 * 1-税邦云
	 */
	@ApiModelProperty(value="用户认证类型")
	private String userType;

	@ApiModelProperty(value="银行卡账号")
	private String bankCardNumber;

	/**
	 * 人力家认证状态
	 */
	@ApiModelProperty(value="人力家认证状态")
	private Integer rljCertStatus;

	/**
	 * 税邦云协议签署,0->未签署，1->已签署
	 */
	@ApiModelProperty(value="税邦云协议签署")
	private Integer taxHelperAgreementSignature;

}
