/*
 *
 *      Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the pig4cloud.com developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: hnqz
 *
 */

package com.qunzhixinxi.hnqz.admin.api.vo;

import com.qunzhixinxi.hnqz.admin.api.entity.SysRole;
import com.qunzhixinxi.hnqz.admin.api.enums.SignatureStatusEnum;
import com.qunzhixinxi.hnqz.common.core.sensitive.Sensitive;
import com.qunzhixinxi.hnqz.common.core.sensitive.SensitiveTypeEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author hnqz
 * @date 2017/10/29
 */
@Data
@ApiModel(value = "前端用户展示对象")
public class UserVO implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * 主键ID
	 */
	@ApiModelProperty(value = "主键")
	private Integer userId;

	/**
	 * 用户名
	 */
	@ApiModelProperty(value = "用户名")
	private String username;

	/**
	 * 身份证号
	 */
	@ApiModelProperty(value = "身份证号")
	private String idCardNumber;

	/**
	 * 密码
	 */
	@ApiModelProperty(value = "密码")
	private String password;

	/**
	 * 随机盐
	 */
	@ApiModelProperty(value = "随机盐")
	private String salt;

	/**
	 * 微信openid
	 */
	@ApiModelProperty(value = "微信open id")
	private String wxOpenid;

	/**
	 * QQ openid
	 */
	@ApiModelProperty(value = "qq open id")
	private String qqOpenid;

	/**
	 * 创建时间
	 */
	@ApiModelProperty(value = "创建时间")
	private LocalDateTime createTime;

	/**
	 * 修改时间
	 */
	@ApiModelProperty(value = "修改时间")
	private LocalDateTime updateTime;

	/**
	 * 0-正常，1-删除
	 */
	@ApiModelProperty(value = "删除标记,1:已删除,0:正常")
	private String delFlag;

	/**
	 * 锁定标记
	 */
	@ApiModelProperty(value = "锁定标记,0:正常,9:已锁定")
	private String lockFlag;

	/**
	 * 手机号
	 */
	@Sensitive(type = SensitiveTypeEnum.MOBILE_PHONE)
	@ApiModelProperty(value = "手机号")
	private String phone;

	/**
	 * 头像
	 */
	@ApiModelProperty(value = "头像")
	private String avatar;

	/**
	 * 部门ID
	 */
	@ApiModelProperty(value = "所属部门")
	private Integer deptId;

	/**
	 * 租户ID
	 */
	@ApiModelProperty(value = "所属租户")
	private Integer tenantId;

	/**
	 * 部门名称
	 */
	@ApiModelProperty(value = "所属部门名称")
	private String deptName;


	/**
	 * 用户姓名
	 */
	@ApiModelProperty(value="用户姓名")
	private String realname;
	/**
	 * 省
	 */
	@ApiModelProperty(value="省")
	private String province;
	private String provinceCode;
	/**
	 * 市
	 */
	@ApiModelProperty(value="市")
	private String city;
	private String cityCode;
	/**
	 * 区
	 */
	@ApiModelProperty(value="区")
	private String area;
	/**
	 * 地址
	 */
	@ApiModelProperty(value="地址")
	private String address;
	/**
	 * 备用字段1
	 */
	@ApiModelProperty(value="备用字段1")
	private String temp1;
	/**
	 * 备用字段2
	 */
	@ApiModelProperty(value="备用字段2")
	private String temp2;
	/**
	 * 备用字段3
	 */
	@ApiModelProperty(value="备用字段3")
	private String temp3;
	/**
	 * 备用字段4
	 */
	@ApiModelProperty(value="备用字段4")
	private String temp4;
	/**
	 * 备用字段5
	 */
	@ApiModelProperty(value="备用字段5")
	private String temp5;

	/**
	 * 备用字段5
	 */
	@ApiModelProperty(value="平台ID")
	private String platId;


	/**
	 * 备用字段5
	 */
	@ApiModelProperty(value="药企ID")
	private String drugEntId;

	/**
	 * 角色列表
	 */
	@ApiModelProperty(value = "拥有的角色列表")
	private List<SysRole> roleList;

	/**
	 * 备用字段5
	 */
	@ApiModelProperty(value="w1")
	private String w1;

	/**
	 * 人力家认证状态
	 */
	@ApiModelProperty(value="税邦云认证状态")
	private Integer certStatus;

	private String certRemark;

	private String userSign;

	private String userType;

	private String bankCardNumber;

	/**
	 * 人力家认证状态
	 */
	@ApiModelProperty(value="人力家认证状态")
	private Integer rljCertStatus;

	/**
	 * 税邦云协议签署,0->未签署，1->已签署
	 */
	@ApiModelProperty(value="税邦云协议签署")
	private Integer taxHelperAgreementSignature;

}
