/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.WmFeedback;
import com.qunzhixinxi.hnqz.admin.service.WmFeedbackService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * 意见反馈
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:42
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmfeedback" )
@Api(value = "wmfeedback", tags = "意见反馈管理")
public class WmFeedbackController {

    private final  WmFeedbackService wmFeedbackService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmFeedback 意见反馈
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('admin_wmfeedback_view')" )
    public R getWmFeedbackPage(Page page, WmFeedback wmFeedback) {

		OrderItem orderItem=new OrderItem();
		orderItem.setColumn("create_time");
		orderItem.setAsc(false);
		page.addOrder(orderItem);

        return R.ok(wmFeedbackService.page(page, Wrappers.query(wmFeedback)));
    }


    /**
     * 通过id查询意见反馈
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_wmfeedback_view')" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmFeedbackService.getById(id));
    }

    /**
     * 新增意见反馈
     * @param wmFeedback 意见反馈
     * @return R
     */
    @ApiOperation(value = "新增意见反馈", notes = "新增意见反馈")
    @SysLog("新增意见反馈" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_wmfeedback_add')" )
    public R save(@RequestBody WmFeedback wmFeedback) {
		wmFeedback.setCreateTime(LocalDateTime.now());
    	return R.ok(wmFeedbackService.save(wmFeedback));
    }

    /**
     * 修改意见反馈
     * @param wmFeedback 意见反馈
     * @return R
     */
    @ApiOperation(value = "修改意见反馈", notes = "修改意见反馈")
    @SysLog("修改意见反馈" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_wmfeedback_edit')" )
    public R updateById(@RequestBody WmFeedback wmFeedback) {
        return R.ok(wmFeedbackService.updateById(wmFeedback));
    }

    /**
     * 通过id删除意见反馈
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除意见反馈", notes = "通过id删除意见反馈")
    @SysLog("通过id删除意见反馈" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_wmfeedback_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmFeedbackService.removeById(id));
    }

}
