/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackageTaskRef;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageTaskRefService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * 任务-积分包关联
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:25
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmscorepackagetaskref" )
@Api(value = "wmscorepackagetaskref", tags = "任务-积分包关联管理")
public class WmScorePackageTaskRefController {

    private final  WmScorePackageTaskRefService wmScorePackageTaskRefService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmScorePackageTaskRef 任务-积分包关联
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    public R getWmScorePackageTaskRefPage(Page page, WmScorePackageTaskRef wmScorePackageTaskRef) {
		OrderItem orderItem=new OrderItem();
		orderItem.setColumn("create_time");
		orderItem.setAsc(false);
		page.addOrder(orderItem);

        return R.ok(wmScorePackageTaskRefService.page(page, Wrappers.query(wmScorePackageTaskRef)));
    }


    /**
     * 通过id查询任务-积分包关联
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmScorePackageTaskRefService.getById(id));
    }

    /**
     * 新增任务-积分包关联
     * @param wmScorePackageTaskRef 任务-积分包关联
     * @return R
     */
    @ApiOperation(value = "新增任务-积分包关联", notes = "新增任务-积分包关联")
    @SysLog("新增任务-积分包关联" )
    @PostMapping
    public R save(@RequestBody WmScorePackageTaskRef wmScorePackageTaskRef) {
		wmScorePackageTaskRef.setCreateTime(LocalDateTime.now());
        return R.ok(wmScorePackageTaskRefService.save(wmScorePackageTaskRef));
    }

    /**
     * 修改任务-积分包关联
     * @param wmScorePackageTaskRef 任务-积分包关联
     * @return R
     */
    @ApiOperation(value = "修改任务-积分包关联", notes = "修改任务-积分包关联")
    @SysLog("修改任务-积分包关联" )
    @PutMapping
    public R updateById(@RequestBody WmScorePackageTaskRef wmScorePackageTaskRef) {
        return R.ok(wmScorePackageTaskRefService.updateById(wmScorePackageTaskRef));
    }

    /**
     * 通过id删除任务-积分包关联
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除任务-积分包关联", notes = "通过id删除任务-积分包关联")
    @SysLog("通过id删除任务-积分包关联" )
    @DeleteMapping("/{id}" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmScorePackageTaskRefService.removeById(id));
    }

}
