/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskTypeScoreRef;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeScoreRefService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * 任务类型打分
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:25
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmtasktypescoreref" )
@Api(value = "wmtasktypescoreref", tags = "任务类型打分管理")
public class WmTaskTypeScoreRefController {

    private final  WmTaskTypeScoreRefService wmTaskTypeScoreRefService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmTaskTypeScoreRef 任务类型打分
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
	//@PreAuthorize("@pms.hasPermission('admin_wmtasktypescoreref_view')" )
    public R getWmTaskTypeScoreRefPage(Page page, WmTaskTypeScoreRef wmTaskTypeScoreRef) {

		OrderItem orderItem=new OrderItem();
		orderItem.setColumn("create_time");
		orderItem.setAsc(false);
		page.addOrder(orderItem);

        return R.ok(wmTaskTypeScoreRefService.page(page, Wrappers.query(wmTaskTypeScoreRef)));
    }


    /**
     * 通过id查询任务类型打分
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
	//@PreAuthorize("@pms.hasPermission('admin_wmtasktypescoreref_view')" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmTaskTypeScoreRefService.getById(id));
    }

    /**
     * 新增任务类型打分
     * @param wmTaskTypeScoreRef 任务类型打分
     * @return R
     */
    @ApiOperation(value = "新增任务类型打分", notes = "新增任务类型打分")
    @SysLog("新增任务类型打分" )
    @PostMapping
	//@PreAuthorize("@pms.hasPermission('admin_wmtasktypescoreref_add')" )
    public R save(@RequestBody WmTaskTypeScoreRef wmTaskTypeScoreRef) {
		wmTaskTypeScoreRef.setCreateTime(LocalDateTime.now());
        return R.ok(wmTaskTypeScoreRefService.save(wmTaskTypeScoreRef));
    }

    /**
     * 修改任务类型打分
     * @param wmTaskTypeScoreRef 任务类型打分
     * @return R
     */
    @ApiOperation(value = "修改任务类型打分", notes = "修改任务类型打分")
    @SysLog("修改任务类型打分" )
    @PutMapping
	//@PreAuthorize("@pms.hasPermission('admin_wmtasktypescoreref_edit')" )
    public R updateById(@RequestBody WmTaskTypeScoreRef wmTaskTypeScoreRef) {
        return R.ok(wmTaskTypeScoreRefService.updateById(wmTaskTypeScoreRef));
    }

    /**
     * 通过id删除任务类型打分
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除任务类型打分", notes = "通过id删除任务类型打分")
    @SysLog("通过id删除任务类型打分" )
    @DeleteMapping("/{id}" )
	//@PreAuthorize("@pms.hasPermission('admin_wmtasktypescoreref_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmTaskTypeScoreRefService.removeById(id));
    }

}
