/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.daemon.quartz.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.WmScorePackageStatus;
import com.qunzhixinxi.hnqz.daemon.quartz.service.WmScorePackageStatusService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 积分包状态
 *
 * @author pigx code generator
 * @date 2020-07-31 20:07:00
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmscorepackagestatus" )
@Api(value = "wmscorepackagestatus", tags = "积分包状态管理")
public class WmScorePackageStatusController {

    private final  WmScorePackageStatusService wmScorePackageStatusService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmScorePackageStatus 积分包状态
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackagestatus_view')" )
    public R getWmScorePackageStatusPage(Page page, WmScorePackageStatus wmScorePackageStatus) {
        return R.ok(wmScorePackageStatusService.page(page, Wrappers.query(wmScorePackageStatus)));
    }


    /**
     * 通过id查询积分包状态
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackagestatus_view')" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmScorePackageStatusService.getById(id));
    }

    /**
     * 新增积分包状态
     * @param wmScorePackageStatus 积分包状态
     * @return R
     */
    @ApiOperation(value = "新增积分包状态", notes = "新增积分包状态")
    @SysLog("新增积分包状态" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackagestatus_add')" )
    public R save(@RequestBody WmScorePackageStatus wmScorePackageStatus) {
        return R.ok(wmScorePackageStatusService.save(wmScorePackageStatus));
    }

    /**
     * 修改积分包状态
     * @param wmScorePackageStatus 积分包状态
     * @return R
     */
    @ApiOperation(value = "修改积分包状态", notes = "修改积分包状态")
    @SysLog("修改积分包状态" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackagestatus_edit')" )
    public R updateById(@RequestBody WmScorePackageStatus wmScorePackageStatus) {
        return R.ok(wmScorePackageStatusService.updateById(wmScorePackageStatus));
    }

    /**
     * 通过id删除积分包状态
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除积分包状态", notes = "通过id删除积分包状态")
    @SysLog("通过id删除积分包状态" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackagestatus_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmScorePackageStatusService.removeById(id));
    }

}
