/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.daemon.quartz.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.WmTask;
import com.qunzhixinxi.hnqz.daemon.quartz.service.WmTaskService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 任务表
 *
 * @author pigx code generator
 * @date 2020-09-27 18:11:25
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmtask" )
@Api(value = "wmtask", tags = "任务表管理")
public class WmTaskController {

    private final  WmTaskService wmTaskService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmTask 任务表
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('quartz_wmtask_view')" )
    public R getWmTaskPage(Page page, WmTask wmTask) {
        return R.ok(wmTaskService.page(page, Wrappers.query(wmTask)));
    }


    /**
     * 通过id查询任务表
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('quartz_wmtask_view')" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmTaskService.getById(id));
    }

    /**
     * 新增任务表
     * @param wmTask 任务表
     * @return R
     */
    @ApiOperation(value = "新增任务表", notes = "新增任务表")
    @SysLog("新增任务表" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('quartz_wmtask_add')" )
    public R save(@RequestBody WmTask wmTask) {
        return R.ok(wmTaskService.save(wmTask));
    }

    /**
     * 修改任务表
     * @param wmTask 任务表
     * @return R
     */
    @ApiOperation(value = "修改任务表", notes = "修改任务表")
    @SysLog("修改任务表" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('quartz_wmtask_edit')" )
    public R updateById(@RequestBody WmTask wmTask) {
        return R.ok(wmTaskService.updateById(wmTask));
    }

    /**
     * 通过id删除任务表
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除任务表", notes = "通过id删除任务表")
    @SysLog("通过id删除任务表" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('quartz_wmtask_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmTaskService.removeById(id));
    }

}
